VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CBinarySearchTree"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'
' Operationen auf binre Bume.
'
' Es ist zu beachten, dass zum Einfgen bzw. Lschen von Knoten aus dem
' Baum nur die hier implementierten Prozeduren 'Insert' und 'Delete'
' verwendet werden sollten, da sonst bei unsachgemssem Entfernen bzw.
' Einfgen die Binrbaumeigenschaft zerstrt werden kann, wodurch die
' Operationen falsche Objekte zurckgeben. Weiters sollten aus diesem
' Grund die Zeiger auf 'LeftSon', 'RightSon' und 'Father' nicht manuell
' verndert werden.
'
Option Explicit

Public Enum EventType
    ePreOrder = 0
    eInOrder = 1
    ePostOrder = 2
    eEnumInterval = 3
End Enum

' Callback fr 'InOrder' und 'GetInterval'.
' Im Parameter 'Node' wird ein Zeiger auf den Knoten bergeben, damit der
' Benutzer der Klasse die Mglichkeit besitzt, die Struktur des Baumes zu
' verndern.
Public Event Traversed(ByVal Node As CNode, ByVal EventSource As EventType)

'***********************************
' Grundlegende Operationen.
'***********************************

'
' Lscht den Knoten 'Node' aus dem Binrbaum mit Wurzel 'Root'.
'
Public Sub Delete(ByRef Root As CNode, ByVal Node As CNode)
    
    ' Bestimme den Knoten 'r', der herausgeschnitten werden soll.
    Dim r As CNode
    If Node.LeftSon Is Nothing Or Node.RightSon Is Nothing Then
        Set r = Node
    Else
        Set r = Successor(Node)
    End If
    
    ' Lasse 'p' auf das Kind von 'r' zeigen, bzw. auf 'Nothing', falls 'r'
    ' keine Kinder hat.
    Dim p As CNode
    If Not r.LeftSon Is Nothing Then
        Set p = r.LeftSon
    Else
        Set p = r.RightSon
    End If
    
    ' Schneide 'r' heraus.
    If Not p Is Nothing Then
        Set p.Father = r.Father
    End If
    If r.Father Is Nothing Then
        Set Root = p
    Else
        If r Is r.Father.LeftSon Then
            Set r.Father.LeftSon = p
        Else
            Set r.Father.RightSon = p
        End If
    End If
    
    ' Kopiere bei Bedarf Daten von 'r' nach 'q'.
    If Not r Is Node Then
        Node.Key = r.Key
        Node.Data = r.Data
    End If
End Sub

'
' Sucht einen Knoten mit Schlssel 'Key' im Baum mit Wurzel 'Root'.
'
Public Function Search(ByVal Root As CNode, ByVal Key As Long) As CNode
    Dim p As CNode
    Set p = Root
    Do While Not p Is Nothing
        If p.Key = Key Then
            Exit Do
        Else
            If Key < p.Key Then
                Set p = Root.LeftSon
            Else
                Set p = Root.RightSon
            End If
        End If
    Loop
    Set Search = p
End Function

'
' Fgt den Knoten 'Node' in den Baum mit Wurzel 'Root' ein.
'
Public Sub Insert(ByRef Root As CNode, ByVal Node As CNode)
    Dim r As CNode
    Set r = Nothing
    Dim p As CNode
    Set p = Root
    Do While Not p Is Nothing
        Set r = p
        If Node.Key < p.Key Then
             Set p = p.LeftSon
        Else
             Set p = p.RightSon
        End If
    Loop
    Set Node.Father = r
    Set Node.LeftSon = Nothing
    Set Node.RightSon = Nothing
    If r Is Nothing Then
        Set Root = Node
    Else
        If Node.Key < r.Key Then
            Set r.LeftSon = Node
        Else
            Set r.RightSon = Node
        End If
    End If
End Sub


'
' bergibt alle Knoten im Baum bzw. Teilbaum mit Wurzel 'Root' in Inorder-
' Durchmusterungsreihenfolge an eine Callbackprozedur.
'
Public Sub InOrder(ByVal Root As CNode)
    Dim p As CNode
    Set p = Root
    If Not p Is Nothing Then
        Call InOrder(p.LeftSon)
        
        ' Ereignis fr Knoten ausfhren.
        RaiseEvent Traversed(p, eInOrder)
        
        ' Folgender auskommentierter Code wrde bei jedem Knoten, der
        ' berlaufen wird, Informationen ber seine Position im Baum
        ' ausgeben.
        '
        'If p.Father Is Nothing Then
        '    Call MsgBox(CStr(p.Key) & " ist Wurzel")
        'Else
        '    If Not p.Father.LeftSon Is Nothing Then
        '        If p.Father.LeftSon Is p Then
        '            Call MsgBox(CStr(p.Key) & " Linker Sohn von " & CStr(p.Father.Key))
        '        Else
        '            Call MsgBox(CStr(p.Key) & " Rechter Sohn von " & CStr(p.Father.Key))
        '        End If
        '    ElseIf Not p.Father.RightSon Is Nothing Then
        '        If p.Father.LeftSon Is p Then
        '            Call MsgBox(CStr(p.Key) & " Linker Sohn von " & CStr(p.Father.Key))
        '        Else
        '            Call MsgBox(CStr(p.Key) & " Rechter Sohn von " & CStr(p.Father.Key))
        '        End If
        '    End If
        'End If
        
        Call InOrder(p.RightSon)
    End If
End Sub

'
' bergibt alle Knoten im Baum bzw. Teilbaum mit Wurzel 'Root' in Preorder-
' Durchmusterungsreihenfolge an eine Callbackprozedur.
'
Public Sub PreOrder(ByVal Root As CNode)
    Dim p As CNode
    Set p = Root
    If Not p Is Nothing Then
        
        ' Ereignis fr Knoten ausfhren.
        RaiseEvent Traversed(p, ePreOrder)
        Call InOrder(p.LeftSon)
        Call InOrder(p.RightSon)
    End If
End Sub

'
' bergibt alle Knoten im Baum bzw. Teilbaum mit Wurzel 'Root' in Postorder-
' Durchmusterungsreihenfolge an eine Callbackprozedur.
'
Private Sub PostOder(ByVal Root As CNode)
    Dim p As CNode
    Set p = Root
    If Not p Is Nothing Then
        Call InOrder(p.LeftSon)
        Call InOrder(p.RightSon)
        
        ' Ereignis fr Knoten ausfhren.
        RaiseEvent Traversed(p, ePostOrder)
    End If
End Sub

'
' Gibt den Knoten mit dem kleinsten Schlssel im Baum mit Wurzel 'Root'
' zurck.
'
Public Function Minimum(ByVal Root As CNode) As CNode
    Dim p As CNode
    Set p = Root
    Do While Not p.LeftSon Is Nothing
        Set p = p.LeftSon
    Loop
    Set Minimum = p
End Function

'
' Gibt den Knoten mit dem grssten Schlssel im Baum mit Wurzel 'Root'
' zurck.
'
Public Function Maximum(ByVal Root As CNode) As CNode
    Dim p As CNode
    Set p = Root
    Do While Not p.RightSon Is Nothing
        Set p = p.RightSon
    Loop
    Set Maximum = p
End Function

Public Function Successor(ByVal Root As CNode) As CNode
    Dim p As CNode
    Set p = Root
    If Not p.RightSon Is Nothing Then
        Set Successor = Minimum(p.RightSon)
    Else
        Dim q As CNode
        Set q = p.Father
        
        ' Wenn 'p' linker Sohn von 'q', dann ist 'q' Successor und wir
        ' sind fertig.
        Do While Not q Is Nothing
            If Not p Is q.RightSon Then
                Exit Do
            Else
                Set p = q
                Set q = q.Father
            End If
        Loop
        Set Successor = q
    End If
End Function

Public Function Predecessor(ByVal Root As CNode) As CNode
    Dim p As CNode
    Set p = Root
    If Not p.LeftSon Is Nothing Then
        Set Predecessor = Maximum(p.LeftSon)
    Else
        Dim q As CNode
        Set q = p.Father
        
        ' Wenn 'p' rechter Sohn von 'q', dann ist 'q' Predecessor und wir
        ' sind fertig.
        Do While Not q Is Nothing
            If Not p Is q.LeftSon Then
                Exit Do
            Else
                Set p = q
                Set q = q.Father
            End If
        Loop
        Set Predecessor = q
    End If
End Function

'***********************************
' Zustzliche Operationen.
'***********************************

'
' bergibt alle Knoten in einem Baum, deren Schlssel in einem angegebenen
' Intervall liegen, aufsteigend sortiert in eine Callbackprozedur.
'
Public Sub EnumInterval(ByVal Root As CNode, ByVal Min As Long, ByVal Max As Long)
    Dim p As CNode
    Set p = Root
    If Not p Is Nothing Then
        
        ' Baum von 'Root' ausgehend in Inorder-Reihenfolge durchlaufen und
        ' Knoten an Callbackprozedur bergeben.
        Call EnumInterval(p.LeftSon, Min, Max)
        If p.Key < Max And p.Key > Min Then
            RaiseEvent Traversed(p, eEnumInterval)
        End If
        Call EnumInterval(p.RightSon, Min, Max)
    End If
End Sub

'
' Gibt den kleinsten Schlssel >= einem bestimmten Wert zurck. Ist kein
' solcher vorhanden, wird 'Nothing' zurckgegeben.
'
Public Function GetLargestNodeLessOrEqual(ByVal Node As CNode, ByVal Key As Long) As CNode
    Dim p As CNode
    Set p = Node
    Do While Not p Is Nothing
        
        ' Schlssel gefunden.
        If p.Key = Key Then
            Set GetLargestNodeLessOrEqual = p
            Exit Function
        
        ' Zu gross, in linkem Teilbaum suchen.
        ElseIf p.Key > Key Then
            Set GetLargestNodeLessOrEqual = p
            Set p = p.LeftSon
        
        ' Sonst (zu klein), im rechten Teilbaum suchen.
        Else
            Set p = p.RightSon
        End If
    Loop
End Function
